;;;============================================================================

;;; File: "debuggee.scm", Time-stamp: <2008-11-08 09:08:33 feeley>

;;; Copyright (c) 2008 by Marc Feeley, All Rights Reserved.

;;;============================================================================

(include "rm.scm")

;;;-----------------------------------------------------------------------------

(define-type special-port
  port
  eof?
)

(define (read-substring-blocking-for-1 str start end sport)
  (cond ((special-port-eof? sport)
         (special-port-eof?-set! sport #f)
         0) ;; reached end-of-file
        ((< start end)
         (let ((port (special-port-port sport)))
           ;; block for the first byte
           (input-port-timeout-set! port +inf.0)
           (let ((n (read-substring str start (+ start 1) port)))
             ;; don't block for the rest
             (if (and (= n 1) (< (+ start 1) end))
                 (let ((to? #f))
                   (input-port-timeout-set! port
                                            -inf.0
                                            (lambda () (set! to? #t) #f))
                   (let ((len (read-substring str (+ start 1) end port)))
                     (if (and (= len 0) (not to?))
                         (special-port-eof?-set! sport #t))
                     (+ n len)))
                 n))))
        (else
         0))) ;; buffer is of length 0!

;;;----------------------------------------------------------------------------

(define rde-ctx #f)
(define rde-connection-ctx #f)

(define (rde-init)
  (set! rde-ctx (rm-ctx-create))
  (set! rde-connection-ctx (rm-connect rde-ctx "localhost" rde-function)))

(define (rde-function connection-ctx name)
  (case name
    ((console-input)
     rde-console-input)
    ((user-interrupt)
     rde-user-interrupt)
    ((get-threads)
     rde-get-threads)
    ((get-extrep)
     rde-get-extrep)
    (else
     (error "unknown function"))))

;;;----------------------------------------------------------------------------

(define rde-console-table (make-table))

(define (rde-get-threads connection-ctx)
  (##tgroup->thread-vector
   (thread-thread-group (##vector-ref (##run-queue) 12))))

(define (rde-get-extrep connection-ctx obj max-length)
  (object->string obj max-length))

(define (rde-user-interrupt connection-ctx console-id)
  (let ((t (serial-number->object console-id)))
    (if t
        (##thread-interrupt! t ##user-interrupt!)))
  #t)

(define (rde-console-input connection-ctx console-id input)
  (let ((remote-port
         (table-ref rde-console-table console-id #f)))
    (if remote-port
        (let ((n (string-length input)))
          (if (= n 0)
              (close-output-port remote-port)
              (begin
                (write-substring input 0 n remote-port)
                (force-output remote-port))))))
  #t)

(define (rde-console-output-pump-start! console-id remote-port)
  (let ((sport (make-special-port remote-port #f)))
    (thread-start!
     (make-thread
      (lambda ()
        (let* ((buflen 1000)
               (buf (make-string buflen)))
          (let loop ()
            (let ((n (read-substring-blocking-for-1 buf 0 buflen sport)))
              (if (> n 0)
                  (begin
                    (rm-remote-call rde-connection-ctx
                                    'console-output
                                    console-id
                                    (substring buf 0 n))
                    (loop)))))))))))

(define (rde-register-console thread remote-port)
  (let ((console-id (object->serial-number thread)))
    (table-set! rde-console-table console-id remote-port)
    (rm-remote-call rde-connection-ctx 'register-console console-id)
    (rde-console-output-pump-start! console-id remote-port)))

(define (make-repl-channel-remote-port thread)
  (receive (local-port remote-port)
           (open-string-pipe (list permanent-close: #f buffering: #f))
    (begin

      ;; Hack... set the name of the port to pretend it is the "console"
      (##vector-set! local-port 4 (lambda (port) '(console)))

      (rde-register-console thread remote-port)
      local-port)))

(define (thread-make-repl-channel-remote thread)
  (let ((local-port (make-repl-channel-remote-port thread)))
    (##make-repl-channel-ports local-port local-port)))

(set! ##thread-make-repl-channel
      thread-make-repl-channel-remote)

(##set-debug-settings! 17 31) ;; simulate -:dar runtime option

(define (start-new-repl)
  (thread-start!
   (make-thread
    (lambda ()
      (##repl-debug-main)))))

;;;----------------------------------------------------------------------------

(rm-setup)
(rde-init)

;;;============================================================================
