;;;============================================================================

;;; File: "debugger.scm", Time-stamp: <2008-11-08 08:59:29 feeley>

;;; Copyright (c) 2008 by Marc Feeley, All Rights Reserved.

;;;============================================================================

(include "rm.scm")

;;;----------------------------------------------------------------------------

(define console-window-num 0)

(define (new-console-window-num)
  (let ((x (+ console-window-num 1)))
    (set! console-window-num x)
    x))

(define (open-console-window)
  (let ((tcp-port (+ 9000 (new-console-window-num))))
    (let ((window
           (open-process
            (list path: "xterm"
                  arguments: (list "-e"
                                   "gsi"
                                   "pump.scm"
                                   (number->string tcp-port))))))
      (let loop ()
        (let ((port
               (with-exception-catcher
                (lambda (e)
                  #f)
                (lambda ()
                  (let ((port (open-tcp-client
                               (list server-address: "localhost"
                                     port-number: tcp-port))))
                    (tcp-client-peer-socket-info port)
                    port)))))
          (if (not port)
              (begin
                (thread-sleep! .1) ;; wait until the pump starts
                (loop))
              port))))))

;;;----------------------------------------------------------------------------

(define rdr-ctx #f)

(define (rdr-init)
  (set! rdr-ctx (rm-ctx-create))
  (rm-server-start rdr-ctx rdr-function))

(define (rdr-function connection-ctx name)
  (case name
    ((register-console)
     rdr-register-console)
    ((console-output)
     rdr-console-output)
    (else
     (error "unknown function"))))

;;;----------------------------------------------------------------------------

(define rdr-console-table (make-table))

(define (rdr-register-console connection-ctx console-id)
  (let ((console-port (open-console-window)))
    (table-set! rdr-console-table console-id console-port)
    (rdr-console-input-pump-start! connection-ctx console-id console-port)
    #f))

(define (rdr-console-output connection-ctx console-id output)
  (let ((console-port
         (table-ref rdr-console-table console-id #f)))
    (if console-port
        (begin
;;          (pretty-print (list 'rdr-console-output output) ##stdout-port)
          (write output console-port)
          (force-output console-port))))
  #t)

(define (rdr-console-input-pump-start! connection-ctx console-id console-port)
  (thread-start!
   (make-thread
    (lambda ()
      (let loop ()
        (let ((x (read console-port)))
          (cond ((equal? x '(user-interrupt))

                 #;
                 (begin
                   (pretty-print '------------ ##stdout-port)
                   (pretty-print
                    (map (lambda (t) (rdr-get-extrep connection-ctx t 100))
                         (vector->list (rdr-get-threads connection-ctx)))
                    ##stdout-port)
                   (pretty-print '------------ ##stdout-port))

                 (rdr-user-interrupt connection-ctx console-id))
                ((equal? x "")
                 (rdr-console-input-eof connection-ctx console-id))
                (else
                 (rdr-console-input connection-ctx console-id x)))
          (loop)))))))

;;;----------------------------------------------------------------------------

;;; Methods callable by debugger.

(define (rdr-console-input connection-ctx console-id text)
  (rm-remote-call connection-ctx 'console-input console-id text))

(define (rdr-console-input-eof connection-ctx console-id)
  (rm-remote-call connection-ctx 'console-input console-id ""))

(define (rdr-user-interrupt connection-ctx console-id)
  (rm-remote-call connection-ctx 'user-interrupt console-id))

(define (rdr-get-threads connection-ctx)
  (rm-remote-call connection-ctx 'get-threads))

(define (rdr-get-extrep connection-ctx obj max-length)
  (rm-remote-call connection-ctx 'get-extrep obj max-length))

;;;----------------------------------------------------------------------------

(rm-setup)
(rdr-init)

;;;============================================================================
