#!

;;; File: "pump.scm"

;; start like this:
;;
;;  % xterm gsi -:tE pump.scm 9000
;;
;; then
;;
;;  % telnet localhost 9000

(define port-num (string->number (cadr (command-line))))

(define listen-port (open-tcp-server (list port-number: port-num reuse-address: #t)))

(define connection (read listen-port))

(define got-user-interrupt #f)

(current-user-interrupt-handler
 (lambda ()
   (set! got-user-interrupt #t)))

(define (copy1 in out)
  (let loop ()
    (let ((buf (read in)))
;;      (pretty-print (list 'copy1 buf) ##stdout-port)
      (if (string? buf)
          (begin
            (write-substring buf 0 (string-length buf) out)
            (force-output out)
            (loop))))))

(define (copy2 in out)
  (let* ((buflen 1000)
         (buf (make-string buflen)))
    (let loop1 ()
      (let loop2 ((i 0))

        (define (send x)
;;          (pretty-print (list 'sending x) ##stdout-port)
          (write x out)
          (force-output out))

        (define (send-buffered)
          (if (> i 0)
              (send (substring buf 0 i))))

        (define (send-end-of-file)
          (send ""))

        (define (send-user-interrupt)
          (send-buffered)
          (send '(user-interrupt)))

        (cond (got-user-interrupt
               (set! got-user-interrupt #f)
               (send-user-interrupt)
               (loop1))
              ((< i buflen)
               (let ((to? #f))
                 (input-port-timeout-set! in 0.05 (lambda () (set! to? #t) #f))
                 (let ((c (read-char in)))
                   (cond ((char? c)
                          (string-set! buf i c)
                          (loop2 (+ i 1)))
                         (to?
                          (send-buffered)
                          (loop1))
                         (else
                          (send-end-of-file)
                          (loop1)))))
               (else
                (send-buffered)
                (loop1))))))))

(let ((in (repl-input-port))
      (out (repl-output-port)))
  (if (tty? in)
      (tty-mode-set! in #t #t #f #f 38400))
  (thread-start!
   (make-thread
    (lambda ()
      (copy2 in connection))))
  (copy1 connection out))
